#! /usr/bin/python
# -*- encoding: utf-8 -*-
from datetime import datetime, timedelta

from django.conf import settings
from rest_framework import serializers
from rest_framework.utils import model_meta
from rest_framework.serializers import ValidationError

from apella.models import ApellaUser, Institution, Department, \
    Position, Professor, InstitutionManager, JiraIssue, UserApplication, \
    Registry, RegistryMembership, Candidate, ForeignDiplomaDetails, \
    ApellaFile, MCAInstitutionPositions, Discipline, \
    MCADepartmentPositions, MCADepartmentPositionsDetails, \
    ElectorDisciplineRegistryMembership, ElectorDisciplineRegistry
from apella import auth_hooks
from apella.util import move_to_timezone, otz, at_day_start, normalize_index_field
from apella.emails import send_user_email, send_create_application_emails
from apella.jira_wrapper import create_issue, update_issue
from apella.validators import validate_max_internal_same_discipline


def get_is_secretary(user):
    if user.is_assistant():
        return user.institutionmanager.is_secretary
    return False

def get_role_id(user):
    if user.is_professor():
        try:
            return user.professor.id
        except Professor.DoesNotExist:
            return 0
    elif user.is_candidate():
        try:
            return user.candidate.id
        except Candidate.DoesNotExist:
            return 0
    elif user.is_manager():
        try:
            return user.institutionmanager.id
        except InstitutionManager.DoesNotExist:
            return 0
    else:
        return user.id


def get_insitution_id(self):
    return self.institution.id


def get_institution_id_from_dep(self):
    return self.department.institution.id


def get_department_id(self):
    return self.department.id


def get_mca_institution_ids(self):
    mcaips = MCAInstitutionPositions.objects.filter(
        mca=self,
    )
    institution_ids = []
    for mcaip in mcaips:
        institution_ids.append(str(mcaip.institution.id))
    return ",".join(institution_ids)


def get_mca_department_ids(self):
    mcadps = MCADepartmentPositions.objects.filter(
        institution_positions=self,
    )

    department_ids = []
    for mcadp in mcadps:
        department_ids.append(str(mcadp.department.id))
    return ",".join(department_ids)


def get_mca_total_positions(self):
    mcaips = MCAInstitutionPositions.objects.filter(
        mca=self,
    )
    total_positions = 0
    for mcaip in mcaips:
        total_positions += int(mcaip.positions)
    return total_positions


def get_mcaip_total_positions(self):
    mcadps = MCADepartmentPositions.objects.filter(
        institution_positions=self,
    )
    total_positions = 0
    for mcadp in mcadps:
        total_positions += int(mcadp.positions)
    return total_positions


def get_mcadp_total_positions(self):
    return MCADepartmentPositionsDetails.objects.filter(
        department_positions=self,
    ).count()


def user_application_cannot_create_position(instance):
    """
    Returns true if there exists at least one not cancelled latest position
    with approved user_application whose user and app_type are the same as
    the instance application or user application is in pending state.
    """
    if instance.state == 'pending' or instance.state == 'rejected':
        return True

    approved_apps = UserApplication.objects.filter(
        user=instance.user,
        app_type=instance.app_type,
        state='approved'
    )
    positions = Position.objects.filter(
        user_application__in=approved_apps
    ).exclude(state='cancelled').exclude(state='failed').exclude(state='successful')
    latest_positions = [x for x in positions if not x.is_historical]
    return len(latest_positions)>0


class ValidatorMixin(object):

    def validate(self, data):
        instance = getattr(self, 'instance')
        if not instance:
            model = self.Meta.model
            instance = model(**data)
        else:
            for attr, val in data.items():
                setattr(instance, attr, val)
        instance.clean()
        return super(ValidatorMixin, self).validate(data)


def create_objects(model, fields, validated_data):
    """
    Recursively creates nested objects.
    """
    for field_name, field in fields.iteritems():
        if isinstance(field, serializers.BaseSerializer) \
                and field_name in validated_data:
            nested_data = validated_data.pop(field_name, None)
            nested_object = create_objects(
                field.Meta.model, field.get_fields(), nested_data)
            validated_data[field_name] = nested_object

    info = model_meta.get_field_info(model)
    many_to_many = {}
    for field_name, relation_info in info.relations.items():
        if relation_info.to_many and (field_name in validated_data):
            many_to_many[field_name] = validated_data.pop(field_name)

    if model == ApellaUser:
        obj = model.objects.create_user(**validated_data)
    else:
        obj = model.objects.create(**validated_data)

    if many_to_many:
        for field_name, value in many_to_many.items():
            setattr(obj, field_name, value)
        obj.save()
    return obj


def update_objects(fields, validated_data, instance=None, model=None):
    """
    Recursively updated nested objects.
    """
    if not instance:
        return create_objects(model, fields, validated_data)
    info = model_meta.get_field_info(type(instance))
    many_to_many = {}
    for field_name, relation_info in info.relations.items():
        if relation_info.to_many and (field_name in validated_data):
            many_to_many[field_name] = validated_data.pop(field_name)

    for field_name, field in fields.iteritems():
        if isinstance(field, serializers.BaseSerializer) \
                and field_name in validated_data:
            nested_data = validated_data.pop(field_name, None)
            nested_object = update_objects(
                field.get_fields(), nested_data,
                instance=getattr(instance, field_name), model=field.Meta.model)
            setattr(instance, field_name, nested_object)
        elif field_name in validated_data:
            if field_name == 'password' or field_name == 'is_upgraded':
                continue
            else:
                setattr(instance, field_name, validated_data.get(field_name))
    if many_to_many:
        for field_name_many, value in many_to_many.items():
            setattr(instance, field_name_many, value)
    instance.save()
    return instance


class NestedWritableObjectsMixin(object):

    NESTED_USER_KEY = 'user'

    def __init__(self, *args, **kwargs):
        super(NestedWritableObjectsMixin, self).__init__(*args, **kwargs)
        request = self.context.get('request')
        if request and \
                request.method in ['GET', 'PUT', 'PATCH'] and \
                self.NESTED_USER_KEY in self.fields:
            self.fields[self.NESTED_USER_KEY].fields['username'].read_only = \
                True
            self.fields[self.NESTED_USER_KEY].fields['email'].read_only = \
                True
            del self.fields[self.NESTED_USER_KEY].fields['password']

    def create(self, validated_data):
        model = self.Meta.model
        obj = create_objects(model, self.get_fields(), validated_data)
        return obj

    def update(self, instance, validated_data):
        instance = update_objects(
            self.get_fields(), validated_data, instance=instance)
        if isinstance(instance, Department):
            empty_positions = Position.objects.filter(
                department=instance.id, department_dep_number=0)
            dep_number = validated_data.get('dep_number', 0)
            for p in empty_positions:
                if dep_number > 0:
                    p.department_dep_number = dep_number
                    p.save()
        return instance


class HelpdeskUsers(object):

    def to_representation(self, obj):
        data = super(HelpdeskUsers, self).to_representation(obj)
        if (obj.is_helpdesk() or obj.is_ministry() or obj.is_yyet()) and \
                self.context['view'].get_view_name() == 'Custom User':
            data = {'user': data}
        return data

    def to_internal_value(self, data):
        user = self.context.get('request').user
        if (user.is_helpdesk() or user.is_ministry() or user.is_yyet()) and \
                self.context['view'].get_view_name() == 'Custom User':
            return self.context.get('request').data.get('user')
        return super(HelpdeskUsers, self).to_internal_value(data)


class Assistants(NestedWritableObjectsMixin):

    def create(self, validated_data):
        user = self.context.get('request').user
        if user.is_institutionmanager():
            validated_data['institution'] = user.institutionmanager.institution
        validated_data['user']['role'] = 'assistant'
        assistant = super(Assistants, self).create(validated_data)
        auth_hooks.verify_email(assistant.user)
        assistant.user.save()
        auth_hooks.verify_user(assistant)
        assistant.save()
        return assistant


SHIBBOLETH_IDP_WHITELIST = getattr(settings, 'SHIBBOLETH_IDP_WHITELIST', [])
class Professors(object):

    def validate_institution(self, value):
        user = self.instance and self.instance.user
        if user and user.shibboleth_idp and \
                not user.shibboleth_idp in SHIBBOLETH_IDP_WHITELIST:
            if not value or (value.idp != user.shibboleth_idp):
                raise ValidationError("invalid.institution")

        if user.shibboleth_idp:
            idp = user.shibboleth_idp
            org = user.shibboleth_schac_home_organization
            insts = Institution.objects.filter(idp=idp)
            if insts.count() > 1:
                if user.shibboleth_schac_home_organization:
                    insts = insts.filter(schac_home_organization=org)
                    if not value in insts:
                        raise ValidationError("invalid.institution")
        return value


def send_registry_emails(members, department, type):
    department_title_el = department.title.el
    department_title_en = department.title.en
    institution_title_el = department.institution.title.el
    institution_title_en = department.institution.title.en
    registry_type_el = 'εσωτερικό' if type == 'internal' else 'εξωτερικό'
    registry_type_en = type

    for professor in members:
        send_user_email(
            professor.user,
            'apella/emails/registry_add_member_to_member_subject.txt',
            'apella/emails/registry_add_member_to_member_body.txt',
            extra_context={
                'department_title_el': department_title_el,
                'department_title_en': department_title_en,
                'institution_title_el': institution_title_el,
                'institution_title_en': institution_title_en,
                'registry_type_el': registry_type_el,
                'registry_type_en': registry_type_en,
            }
        )


class PositionsPortal(object):
    def to_representation(self, obj):
        now = move_to_timezone(datetime.utcnow(), otz)
        starts_at = None
        if obj.starts_at:
            starts_at = move_to_timezone(obj.starts_at, otz)
        ends_at = None
        if obj.ends_at:
            ends_at = move_to_timezone(obj.ends_at, otz)

        state_el = ''
        if obj.state == 'posted' and starts_at < now:
            state_el = u'Ανοιχτή'
        elif obj.state == 'posted' and starts_at >= now:
            state_el = u'Ενταγμένη'

        data = {
                'id': obj.code,
                'name': obj.title,
                'description': obj.description,
                'department': {
                    'id': obj.department.id,
                    'name': {
                        'el': obj.department.title.el,
                        'en': obj.department.title.en
                    },
                    'school': {
                        'id': obj.department.school and obj.department.school.id,
                        'name': {
                            'el': obj.department.school and obj.department.school.title.el,
                            'en': obj.department.school and obj.department.school.title.en
                        },
                        'institution': {
                            'id': obj.department.institution.id,
                            'name': {
                                'el': obj.department.institution.title.el,
                                'en': obj.department.institution.title.en
                            },
                            'schacHomeOrganization':
                                obj.department.institution.schac_home_organization,
                            'category': obj.department.institution.category
                        }
                    }
                },
                'subject': {
                    'name': obj.discipline
                },
                'sector': {
                    'areaId': obj.subject_area.id,
                    'subjectId': obj.subject.id,
                    'name': {
                        'el': {
                            'area': obj.subject_area.title.el,
                            'subject': obj.subject.title.el
                        },
                        'en': {
                            'area': obj.subject_area.title.en,
                            'subject': obj.subject.title.en
                        }
                    }
                },
                'fek': obj.fek,
                'phase': {
                    'status': obj.state,
                    'clientStatusInGreek': state_el,
                    'clientStatus': obj.state,
                    'candidacies': {
                        'openingDate': starts_at and starts_at.date(),
                        'closingDate': ends_at and ends_at.date(),
                        'createdAt': obj.created_at and obj.created_at.date(),
                        'updatedAt': obj.updated_at and obj.updated_at.date()
                    },
                    'createdAt': obj.created_at and obj.created_at.date(),
                    'updatedAt': obj.updated_at and obj.updated_at.date()
                },
                'fekSentDate': obj.fek_posted_at and obj.fek_posted_at.date()
        }
        return data


class UserApplications(object):
    def validate(self, data):
        app_type = self.instance and self.instace.app_type \
            or data.get('app_type')
        if app_type == 'move' and not data.get('receiving_department', None):
            raise ValidationError('receiving_department.required')

        user = data.get('user', None)
        if not user:
            user = self.context.get('request').user
        if UserApplication.objects.filter(
                user=user,
                app_type=app_type,
                state='pending').exists():
            raise ValidationError('cannot.create.application')

        approved_apps = UserApplication.objects.filter(
            user=user,
            app_type=app_type,
            state='approved')
        for app in approved_apps:
            positions = app.position_set.all()
            for p in positions:
                if p.state != 'cancelled' and p.state != 'failed' \
                        and p.state !='successful' \
                        and not p.is_historical:
                    raise ValidationError('cannot.create.application')
        return super(UserApplications, self).validate(data)

    def create(self, validated_data):
        user = validated_data.get('user', None)
        if not user:
            user = self.context.get('request').user
            if user.is_helpdesk():
                raise ValidationError("user.missing")
            validated_data['user'] = user
        try:
            department = user.professor.department
            validated_data['department'] = department
        except Professor.DoesNotExist:
            raise ValidationError("user.not.a.professor")
        except Department.DoesNotExist:
            raise ValidationError("invalid.department")

        if validated_data.get('app_type', '') == 'move' and \
                self.context.get('request').user.is_helpdesk():
            validated_data['state'] = 'approved'

        obj = super(UserApplications, self).create(validated_data)
        send_create_application_emails(obj)

        return obj

    def update(self, instance, validated_data):
        validated_data['updated_at'] = datetime.utcnow()
        return super(UserApplications, self).update(instance, validated_data)


class InstitutionManagersMixin(object):
    def validate(self, data):
        instance = self.instance
        curr_institution = instance and instance.institution
        new_institution = data.get('institution', None)
        manager_exists = new_institution and \
                InstitutionManager.objects.filter(
                    manager_role='institutionmanager',
                    is_verified=True,
                    institution=new_institution).exists()

        msg = 'manager.exists'
        if instance:
            if new_institution.id != curr_institution.id and \
                    manager_exists:
                raise ValidationError(msg)
        else:
            if manager_exists:
                raise ValidationError(msg)

        return data


class JiraIssues(object):
    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        if isinstance(instance, JiraIssue):
            update_issue(instance)
        elif instance:
            for i in instance:
                update_issue(i)
        return super(JiraIssues, self).__init__(*args, **kwargs)

    def create(self, validated_data):
        jira_issue = JiraIssue(**validated_data)
        new_issue = create_issue(jira_issue)
        validated_data['issue_key'] = new_issue.key
        return super(JiraIssues, self).create(validated_data)

    def validate(self,data):
        request_user = self.context.get('request').user
        if data['reporter'] != request_user:
            raise ValidationError('cannot.create.application')
        if not request_user.is_helpdesk() and data['user'] != request_user:
            raise ValidationError('cannot.create.application')
        return super(JiraIssues, self).validate(data)


class RegistryMembers(object):
    def create(self, validated_data):
        data = self.context.get('request').data
        professor_id = data.get('professor_id', None)
        registry_id = data.get('registry_id', None)
        if not professor_id:
            raise ValidationError("professor.id.required")
        if not registry_id:
            raise ValidationError("registry.id.required")

        try:
            professor = Professor.objects.get(id=professor_id)
        except Professor.DoesNotExist:
            raise ValidationError("professor.not.found")

        try:
            registry = Registry.objects.get(id=registry_id)
        except Registry.DoesNotExist:
            raise ValidationError("registry.not.found")

        if RegistryMembership.objects.filter(
                professor=professor,
                registry=registry).exists():
            raise ValidationError("already.in.registry")

        other_type = 'external' if registry.type == 'internal' \
            else 'internal'
        try:
            other_registry = Registry.objects.get(
                department=registry.department,
                type=other_type)
            if RegistryMembership.objects.filter(
                    registry=other_registry,
                    professor=professor).exists():
                raise ValidationError("already.in.other.registry")

        except Registry.DoesNotExist:
            pass

        rm = RegistryMembership.objects.create(
            professor=professor, registry=registry)

        send_registry_emails([professor], registry.department, registry.type)
        return rm


class ElectorDisciplineRegistryMembers(object):
    def create(self, validated_data):
        data = self.context.get('request').data
        professor_id = data.get('professor_id', None)
        registry_id = data.get('registry_id', None)
        same_discipline = data.get('same_discipline', None)

        if not professor_id:
            raise ValidationError("professor.id.required")
        if not registry_id:
            raise ValidationError("registry.id.required")

        try:
            professor = Professor.objects.get(id=professor_id)
        except Professor.DoesNotExist:
            raise ValidationError("professor.not.found")

        try:
            registry = ElectorDisciplineRegistry.objects.get(id=registry_id)
        except ElectorDisciplineRegistry.DoesNotExist:
            raise ValidationError("registry.not.found")

        if registry.is_active or registry.deactivated_at:
            raise ValidationError("action.not.allowed.registry.status")

        if ElectorDisciplineRegistryMembership.objects.filter(
                professor=professor,
                registry=registry).exists():
            raise ValidationError("already.in.registry")

        if same_discipline is not None:
            if same_discipline == True and registry.registry_type == 'internal':
                if not validate_max_internal_same_discipline(professor):
                    raise ValidationError("max.limit.internal.same.discipline")

        rm = ElectorDisciplineRegistryMembership.objects.create(
            professor=professor,
            registry=registry,
            same_discipline=same_discipline
        )

        return rm

    def update(self, instance, validated_data):
        registry = instance.registry
        if registry.is_active or registry.deactivated_at:
            raise ValidationError("action.not.allowed.registry.status")

        same_discipline = validated_data.get('same_discipline', None)
        if same_discipline is not None:
            if same_discipline == True and registry.registry_type == 'internal':
                professor = instance.professor
                if not validate_max_internal_same_discipline(professor):
                    raise ValidationError("max.limit.internal.same.discipline")
            instance.same_discipline = same_discipline
            instance.save()
        return instance

class ForeignDiplomaDetailsMixin(ValidatorMixin):

    def create(self, validated_data):
        data = self.context.get('request').data
        user = self.context.get('request').user

        file_id = data.get('file_id', None)
        if not file_id:
            raise ValidationError("file.id.missing")

        foreign_institution = validated_data.get('foreign_institution', None)
        if not foreign_institution:
            raise ValidationError("foreign.institution.missing")
        foreign_diploma_type = validated_data.get('foreign_diploma_type', None)

        fdd = ForeignDiplomaDetails.objects.create(
            foreign_institution=foreign_institution,
            foreign_diploma_type=foreign_diploma_type,
            owner=user
        )
        foreign_diploma_file = ApellaFile.objects.get(id=int(file_id))
        foreign_diploma_file.foreign_diploma_details = fdd
        foreign_diploma_file.save()
        return fdd

    def update(self, instance, validated_data):
        foreign_institution = validated_data.get('foreign_institution', None)
        if not foreign_institution:
            raise ValidationError("foreign.institution.missing")
        foreign_diploma_type = validated_data.get('foreign_diploma_type', None)

        instance.foreign_institution = foreign_institution
        instance.foreign_diploma_type = foreign_diploma_type

        instance.save()
        return instance


class DisciplinesMixin(object):

    def create(self, validated_data):
        department = validated_data.get("department", None)
        if not department:
            raise ValidationError("department.missing")
        discipline_text = validated_data.get("discipline_text", None)
        if not discipline_text:
            raise ValidationError("discipline_text.missing")
        scientific_field = validated_data.get("scientific_field", None)
        if not scientific_field:
            raise ValidationError("scientific_field.missing")

        discipline_text_el_index = normalize_index_field(scientific_field["el"]) \
                + "::" + normalize_index_field(discipline_text["el"])
        existing_disciplines = Discipline.objects.filter(
                department=department,
                discipline_text_el_index=discipline_text_el_index)
        if existing_disciplines:
            return existing_disciplines[0]
        else:
            discipline = create_objects(Discipline, self.get_fields(), validated_data)
            discipline.discipline_text_el_index = discipline_text_el_index
            discipline.save()
            return discipline


class DisciplineRegistriesMixin(object):

    def create(self, validated_data):
        department = validated_data.get("department", None)
        if not department:
            raise ValidationError("department.missing")

        department_has_draft_registry = department.disciplineregistry_set.filter(
            activated_at=None, deactivated_at=None).exists()
        if department_has_draft_registry:
            raise ValidationError("department.already.has.a.draft")

        discipline_registry = super(
            DisciplineRegistriesMixin, self).create(validated_data)

        # copy disciplines from active registry
        if department.active_discipline_registry:
            for discipline in department.active_discipline_registry.disciplines.all():
                discipline_registry.disciplines.add(discipline)
        return discipline_registry

    def update(self, instance, validated_data):
        active_from = validated_data.get('active_from', None)
        instance.active_from = at_day_start(active_from, otz) if active_from \
            else None

        active_to = validated_data.get('active_to', None)
        instance.active_to = at_day_start(active_to, otz) if active_to \
            else None
        if active_from and active_to:
            if active_from > active_to:
                raise ValidationError("active.from.greater.than.active.to")
        disciplines = validated_data.get('disciplines', None)
        instance.disciplines.clear()
        if disciplines:
            for discipline in disciplines:
                instance.disciplines.add(discipline)
        instance.save()
        return instance


class ElectorDisciplineRegistriesMixin(object):
    def create(self, validated_data):
        department = validated_data.get("department", None)
        if not department:
            raise ValidationError("department.missing")

        discipline = validated_data.get("discipline", None)
        if not discipline:
            raise ValidationError("discipline.missing")

        registry_type = validated_data.get("registry_type", None)
        if not registry_type:
            raise ValidationError("registry_type.missing")

        department_has_draft_registry = department.electordisciplineregistry_set.filter(
            activated_at=None,
            deactivated_at=None,
            discipline=discipline,
            registry_type=registry_type).exists()
        if department_has_draft_registry:
            raise ValidationError("department.already.has.a.draft.elector.discipline.registry")

        department_active_registry = department.electordisciplineregistry_set.filter(
            discipline=discipline,
            registry_type=registry_type,
            is_active=True).first()

        instance = super(ElectorDisciplineRegistriesMixin, self).create(validated_data)

        if department_active_registry:
            active_members = department_active_registry. \
                electordisciplineregistrymembership_set.all()
            for membership in active_members:
                ElectorDisciplineRegistryMembership.objects.create(
                    registry=instance,
                    professor=membership.professor,
                    same_discipline=membership.same_discipline)

        return instance

    def update(self, instance, validated_data):
        active_from = validated_data.get('active_from', None)
        instance.active_from = at_day_start(active_from, otz) if active_from \
            else None

        active_to = validated_data.get('active_to', None)
        instance.active_to = at_day_start(active_to, otz) if active_to \
            else None
        if active_from and active_to:
            if active_from > active_to:
                raise ValidationError("active.from.greater.than.active.to")

        instance.save()
        return instance


class MCAInstitutionPositionsMixin(object):

    def create(self, validated_data):
        mca = validated_data.get("mca", None)
        institution = validated_data.get("institution", None)
        if not mca:
            raise ValidationError("mca.missing")
        if not institution:
            raise ValidationError("institution.missing")
        mcaip = MCAInstitutionPositions.objects.filter(
            mca=mca,
            institution=institution
        )
        if mcaip:
            raise ValidationError("mca.institution.position.exists")
        return super(MCAInstitutionPositionsMixin, self).create(validated_data)


class MCADepartmentPositionsMixin(object):

    def create(self, validated_data):
        institution_positions = validated_data.get(
            "institution_positions", None)
        if not institution_positions:
            raise ValidationError("institution_positions.missing")

        department = validated_data.get("department", None)
        if not department:
            raise ValidationError("department.missing")

        positions = validated_data.get("positions", None)
        if not positions:
            raise ValidationError("positions.missing")

        user = self.context.get('request').user
        if user.is_institutionmanager():
            if department.institution != user.institutionmanager.institution:
                raise ValidationError("department.not.allowed")

        mcadp = MCADepartmentPositions.objects.filter(
            institution_positions=institution_positions,
            department=department
        ).exists()
        if mcadp:
            raise ValidationError("mca.department.position.exists")

        total_count = get_mcaip_total_positions(institution_positions)
        if total_count + positions > institution_positions.positions:
            raise ValidationError("max.institution.limit.reached")

        return super(MCADepartmentPositionsMixin, self).create(validated_data)

    def update(self, instance, validated_data):
        user = self.context.get('request').user
        if user.is_assistant():
            return super(MCADepartmentPositionsMixin, self).update(instance, validated_data)

        institution_positions = validated_data.get(
            "institution_positions", None)
        if not institution_positions:
            raise ValidationError("institution_positions.missing")

        positions = validated_data.get("positions", None)
        if positions is None:
            raise ValidationError("positions.missing")

        total_count = get_mcaip_total_positions(institution_positions)
        if total_count - instance.positions + positions > institution_positions.positions:
            raise ValidationError("max.institution.limit.reached")

        if instance.mcadepartmentpositionsdetails_set.count() > positions:
            raise ValidationError("more.positions.already.assigned")

        return super(MCADepartmentPositionsMixin, self).update(instance, validated_data)


class MCADepartmentPositionsDetailsMixin(object):

    def create(self, validated_data):
        user = self.context.get('request').user
        if user.is_assistant():
            discipline = validated_data.get("discipline", None)
            department_positions = validated_data.get(
                "department_positions", None)
            if not department_positions:
                raise ValidationError("department_positions.missing")
            department_total_count = get_mcadp_total_positions(department_positions)
            if department_total_count == department_positions.positions:
                raise ValidationError("max.department.limit.reached")

            serial = validated_data.get(
                "serial", None)
            if not discipline:
                raise ValidationError("discipline.missing")
            if not serial:
                raise ValidationError("serial.missing")

            if MCADepartmentPositionsDetails.objects.filter(
                department_positions=department_positions,
                serial=serial,
            ).exists():
                raise ValidationError(
                    "department_positions.serial.combination.exists")

            active_discipline_registry = \
                department_positions.department.active_discipline_registry
            if (
                not active_discipline_registry
                or not active_discipline_registry.disciplines
                or discipline not in active_discipline_registry.disciplines.all()
            ):
                raise ValidationError(
                    "mcadp.discipline.not.in.active_discipline_registry")

        return super(MCADepartmentPositionsDetailsMixin, self).create(validated_data)
